@@ -333,13 +333,3 @@ class ValidSplit:
     def __repr__(self):
         # pylint: disable=useless-super-delegation
         return super(ValidSplit, self).__repr__()
-
-# TODO remove in skorch 0.13
-class CVSplit(ValidSplit):
-    def __init__(self, *args, **kwargs):
-        warnings.warn(
-            f"{self.__class__.__name__} is deprecated, use the new name ValidSplit instead",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        super().__init__(*args, **kwargs)
